class campaignInfo {
    string campaignId;
    string name;
    string cleanName;
    mapInfo@[] maps;
    int mapsLoading = 0;


    campaignInfo() {}

    campaignInfo(string _campaignId, string _name, mapInfo@[] _maps = {}) {
        campaignId = _campaignId;
        name = _name;
        maps = _maps;
        getCleanName(name);
    }


    void getCleanName(const string &in s) {
        bool foundCutof = false;
        for (int i = 0; i < s.Length; i++) {
            if (s[i] == " "[0]) continue;
            if (s[i] != "-"[0]) {
                if (!foundCutof) continue;
                cleanName = name.SubStr(i, name.Length - i);
                return;
            }
            foundCutof = true;
        }
        cleanName = name;
    }

    void addMap(mapInfo@ map) {
        maps.InsertLast(map);
        mapsLoading--;
        if (mapsLoading == 0) campaignsLoading--;
    }

    void writeToBuffer(MemoryBuffer@ buf) {
        stringToBuffer(buf, campaignId);
        stringToBuffer(buf, name);
        mapsTB(buf, maps);
    }

    void mapsTB(MemoryBuffer@ buf, mapInfo@[] val) {
        uint16 len = val.Length;
        buf.Write(len);
        for(uint i = 0; i < len; i++) {
            val[i].writeToBuffer(buf);
        }
    }

    void ReloadMaps() {
        maps = {};
        campaignsLoading++;
        Json::Value json = nadeoAPIReq("https://live-services.trackmania.nadeo.live/api/token/club/"+clubId+"/campaign/"+campaignId)["campaign"]["playlist"];
        if (json.GetType() != Json::Type::Array) return;
        mapsLoading = json.Length;
        for (uint i = 0; i < json.Length; i++) {
            gMapsLoader.quickLoadBatch(json[i]["mapUid"], this);
        }
        for (uint i = 0; i < json.Length; i++) {
            addMap(gMapsLoader.quickGetMap(json[i]["mapUid"]));

        }
        memory.Set(campaignId, this);
        filterMaps();
    }

    void playCampaign() {
        selectedTab = menuTabs::mapsByCampaign;
        resetFilters();
        mapsFilter.name = cleanName;
        mapsFilter.nameOn = true;
        filterMaps();
    }

    void getFilterMaps(mapInfo@[]@ filtered, mapFilter@ mf) {
        for (uint i = 0; i < maps.Length; i++) {
            if (mf.filterMap(maps[i])) filtered.InsertLast(maps[i]);
        }
    }


    void DrawTableRow() {
        UI::PushID(campaignId);
        UI::TableNextRow();
        UI::TableNextColumn();
        UI::PushFont(MidFont);
        UI::Text(cleanName);
        UI::TableNextColumn();
        DrawMedals();
        UI::TableNextColumn();

        if (UI::Button(Icons::Repeat)) {
            startnew(CoroutineFunc(ReloadMaps));
        }

        UI::SameLine();
        if (UI::Button(Icons::Play)) {
            playCampaign();
        }

        UI::PopFont();
        UI::PopID();
    }


    uint[] countMedals() {
        uint[] Medals = array<uint>(6, 0);
        for (uint i = 0; i < maps.Length; i++) {
            uint m = maps[i].getMedal();
            if (m == 0) Medals[0]++;
            for (uint j = 1; j < m+1; j++) {
                Medals[j]++;
            }
        }
        return Medals;
    }

    void DrawMedals(bool sameLine = true, bool reverse = false) {
        uint[] Medals = countMedals();
        uint max = maps.Length;
        int j = (reverse ? 5 : 0);
        for (int i = (reverse ? 5 : 0); (reverse ? i > -1 : i < 6); (reverse ? i-- : i++)) {
            UI::Text(getMedalIcon(i) + Medals[i] + "/" + max);
            if (sameLine) UI::SameLine();
        }
    }

    void drawCampaignButton(vec2 btnSize) {
        vec4 btnColor = vec4(0, 0, 0, 0);
        for (uint i = 0; i < maps.Length; i++) {
            btnColor += getMedalColor(maps[i].getMedal());
        }
        if (maps.Length != 0) btnColor /= maps.Length;
        UI::PushStyleColor(UI::Col::Button, btnColor);
        UI::PushStyleColor(UI::Col::ButtonHovered, btnColor*1.1);
        UI::PushStyleColor(UI::Col::ButtonActive, btnColor*0.9);
        
        if (UI::Button(cleanName, btnSize)) {
            playCampaign();
        }
        if (UI::IsItemHovered()) {
            UI::BeginTooltip();
            DrawMedals(false, true);
            UI::EndTooltip();
        }

        UI::PopStyleColor(3);
    }
}

namespace campaignInfo {
    campaignInfo@ readFromBuffer(MemoryBuffer@ buf) {
        string campaignId = StringFromBuffer(buf);
        string name = StringFromBuffer(buf);
        mapInfo@[] maps = mapsFB(buf);
        return campaignInfo(campaignId, name, maps);
    }

    mapInfo@[] mapsFB(MemoryBuffer@ buf) {
        uint16 len = uint16FromBuffer(buf);
        mapInfo@[] val = {};
        for (uint i = 0; i < len; i++) {
            val.InsertLast(mapInfo::readFromBuffer(buf));
        }
        return val;
    }
}