mem@ memory = mem();
mapWatcher@ mapWatch = mapWatcher();
string pluginVersion = Meta::ExecutingPlugin().Version;
uint[] pluginVersionArr;
bool mapsLoaded = false;
const string clubId = "32195"; // royal training maps club id
const string clubName = "Royal_Training_Maps"; // royal training maps club name
string accountId;
string[] orderedKeys;
const uint16 dataBaseVersion = 10;
mapFilter mapsFilter;
uint campaignAmmount = 0;
uint campaignsLoading = 0;

string windowTitle = "\\$f0f"+Icons::CalendarCheckO +"\\$z "+ "Royal Hub";

bool showWindow = false;

mapsLoader@ gMapsLoader = mapsLoader();

const uint8 activePoolAmmount = 20; // ammount of active royal maps in the current mappool

void Main() {
    prepPluginVersion();
    accountId = NadeoServices::GetAccountID();
    gMapsLoader.startUp();
    memory.LoadMem();
    while (!memory.initialized) yield();
    startnew(initiateNews);
    loadMaps();
    while (!mapsLoaded) yield();
    filterMaps();
}

void prepPluginVersion() {
    string[] temp = pluginVersion.Split('.');
    for (uint i = 0; i < temp.Length; i++) {
        pluginVersionArr.InsertLast(Text::ParseUInt(temp[i]));
    }
}

void RenderMenu() {
    if (UI::MenuItem(windowTitle, "", showWindow)) {
        showWindow = !showWindow;
    }
}

void RenderInterface() {
    if (!showWindow) return;
    vec2 size = vec2(900, 650);
    vec2 pos = (vec2(Draw::GetWidth(), Draw::GetHeight()) - size) / 2.;
    UI::SetNextWindowSize(int(size.x), int(size.y), UI::Cond::FirstUseEver);
    UI::SetNextWindowPos(int(pos.x), int(pos.y), UI::Cond::FirstUseEver);
    UI::PushStyleColor(UI::Col::FrameBg, vec4(.2, .2, .2, .5));
    if (UI::Begin(windowTitle, showWindow)) {
        renderInnerWindow();
    }
    UI::End();
    UI::PopStyleColor();
}

Json::Value@ nadeoAPIReq(const string &in apireq) {
    NadeoServices::AddAudience("NadeoLiveServices");
    while (!NadeoServices::IsAuthenticated("NadeoLiveServices")) yield();
    auto req = NadeoServices::Get("NadeoLiveServices", apireq);
    req.Start();
    while(!req.Finished()) { yield(); }
    if (req.ResponseCode() == 0) {
        yield();
        return nadeoAPIReq(apireq);
    }
    if (develepor) trace("nadeo api req Live " + req.String());
    return Json::Parse(req.String());
}

Json::Value@ nadeoCoreAPIReq(const string &in apireq) {
    NadeoServices::AddAudience("NadeoServices");
    while (!NadeoServices::IsAuthenticated("NadeoServices")) yield();
    auto req = NadeoServices::Get("NadeoServices", apireq);
    req.Start();
    while(!req.Finished()) { yield(); }
    if (req.ResponseCode() == 0) {
        yield();
        return nadeoCoreAPIReq(apireq);
    }
    if (develepor){ 
        trace("nadeo api req core " + req.String());
        if (req.String() == "") trace("response code: " + req.ResponseCode() + "NO RESPONS TO: " + apireq);
    }
    return Json::Parse(req.String());
}


void loadMaps() {
    uint inc = 250;
    orderedKeys = {};
    for (uint j = 0; j < 2000; j += inc) {
        Json::Value json = nadeoAPIReq("https://live-services.trackmania.nadeo.live/api/token/club/"+clubId+"/activity?length="+inc+"&offset="+j+"&active=true")["activityList"];
        for (uint i = 0; i < json.Length; i++) {
            campaignAmmount++;
            auto ci = loadOneCampaign(json[i], i+j);
            memory.Set(ci.campaignId, ci);
            orderedKeys.InsertLast(ci.campaignId);
        }
        
        yield();
    }
    debug("all Campaigns loaded");
    debug("campaing ammount " + campaignAmmount);
}

campaignInfo@ loadOneCampaign(Json::Value json, uint idx, bool forcedLoad = false) {
    string activityType = json["activityType"];
    if (activityType != "campaign") return campaignInfo();
    string Id = tostring(uint32(json["campaignId"]));
    if (memory.In(Id) && idx > 20 && !forcedLoad) return memory.Get(Id);
    string name = json["name"];
    campaignInfo@ ci = campaignInfo(Id, name);
    startnew(CoroutineFunc(ci.ReloadMaps));
    return ci;
}

mapInfo@ loadOneMap(Json::Value json, const string &in campaignId) {
    string trackUid = json["mapUid"];
    Json::Value jsonMap = nadeoAPIReq("https://live-services.trackmania.nadeo.live/api/token/map/"+trackUid);
    string trackId = jsonMap["mapId"];
    string author = jsonMap["author"];
    Json::Value jsonCoreRecords = nadeoCoreAPIReq("https://prod.trackmania.core.nadeo.online/mapRecords/?accountIdList="+accountId+"&mapIdList="+trackId);
    Json::Value jsonCoreMap = nadeoCoreAPIReq("https://prod.trackmania.core.nadeo.online/maps/" + trackId);
    string name = jsonMap["name"];
    string fileUrl = jsonCoreMap["fileUrl"];
    uint64[] medals = {jsonMap["authorTime"], jsonMap["goldTime"], jsonMap["silverTime"], jsonMap["bronzeTime"]};
    uint64 time = 0;
    if (jsonCoreRecords.Length != 0) time = jsonCoreRecords[0]["recordScore"]["time"];
    return mapInfo(trackId, trackUid, campaignId, name, author, fileUrl, medals, time);
}

const string iconUnplayed = "\\$444" + Icons::CircleO + " \\$z";
const string iconPlayed = "\\$444" + Icons::Circle + " \\$z";
const string iconBronze = "\\$964" + Icons::Circle + " \\$z";
const string iconSilver = "\\$899" + Icons::Circle + " \\$z";
const string iconGold = "\\$db4" + Icons::Circle + " \\$z";
const string iconAuthor = "\\$071" + Icons::Circle + " \\$z";

string getMedalIcon(uint medal) {
    const string[] medals = {iconUnplayed, iconPlayed, iconBronze, iconSilver, iconGold, iconAuthor};
    return medals[medal];
}

const string iconWhite = "\\$FFF" + Icons::Circle + " \\$z";
const string iconGreen = "\\$3C3" + Icons::Circle + " \\$z";
const string iconBlue = "\\$00F" + Icons::Circle + " \\$z";
const string iconRed = "\\$F00" + Icons::Circle + " \\$z";
const string iconBlack = "\\$000" + Icons::Circle + " \\$z";
const string iconOther = "\\$F6F" + Icons::Circle + " \\$z";

string getDifficultyIcon(uint difficulty) {
    const string[] difficultyIcons = {iconWhite, iconGreen, iconBlue, iconRed, iconBlack, iconOther};
    return difficultyIcons[difficulty];
}

vec4 rgb(uint r, uint g, uint b) {
    return vec4(r/255., g/255., b/255., .7);
}

vec4 getMedalColor(uint medal) {
    vec4[] medals = {
        rgb(55, 52, 52), // unplayed
        rgb(173, 158, 158), // played
        rgb(166, 72, 4), // bronze
        rgb(193, 190, 187), // silver
        rgb(255, 191, 0), // gold
        rgb(56, 148, 0) // author
    };
    return medals[medal];
}