class mapInfo {
    string trackId = "";
    string trackUid = "";
    string campaignId = "";
    string name = "";
    string author = "";
    string fileUrl = "";
    uint64[] medals = {0, 0, 0, 0};
    uint64 time = 0;
    uint difficulty = 5; // white, green, blue, red, black, other
    bool isOld = false; 

    mapInfo() {}

    mapInfo(string _trackId, string _trackUid, string _campaignId, string _name, string _author, string _fileUrl, uint64[] _medals, uint64 _time = 0) {
        trackId = _trackId;
        trackUid = _trackUid;
        campaignId = _campaignId;
        name = _name;
        author = _author;
        fileUrl = _fileUrl;
        medals = _medals;
        time = _time;
        difficulty = difficultyByName(name);
    }

    void writeToBuffer(MemoryBuffer@ buf) {
        stringToBuffer(buf, trackId);
        stringToBuffer(buf, trackUid);
        stringToBuffer(buf, campaignId);
        stringToBuffer(buf, name);
        stringToBuffer(buf, author);
        stringToBuffer(buf, fileUrl);
        uint64ArrayToBuffer(buf, medals);
        buf.Write(time);
    }

    uint getMedal() {
        if (time == 0) return 0;
        for (int i = medals.Length-1; i > -1; i--) {
            if (time <= medals[i]) continue;
            return medals.Length-i;
        }
        return 5;
    }

    uint getDifficulty() {
        return difficulty;
    }

    void drawMap() {
        UI::PushID(trackId);
        UI::TableNextRow();
        UI::TableNextColumn();
        UI::Text(name);
        UI::TableNextColumn();
        UI::Text(getMedalIcon(getMedal()));
        UI::TableNextColumn();
        if (time != 0) {
            UI::Text(Time::Format(time));
        }
        UI::TableNextColumn();
        if (UI::Button(Icons::Play)) {
            startnew(CoroutineFunc(playMap));
        }
        if (develepor) {
            UI::SameLine();
            if (UI::Button(Icons::Search)) {
                startnew(CoroutineFunc(debugThisMap));
            }
        }

        UI::PopID();
    }

    void debugThisMap() {
        debugMap(this);
    }

    void playMap() {
        if (!Permissions::PlayLocalMap()) {
            error("User cant play local maps!");
            return;
        }
        auto app = cast<CGameManiaPlanet>(GetApp());
        if (app.Network.PlaygroundClientScriptAPI.IsInGameMenuDisplayed) {
            app.Network.PlaygroundInterfaceScriptHandler.CloseInGameMenu(CGameScriptHandlerPlaygroundInterface::EInGameMenuResult::Quit);
        }
        app.BackToMainMenu();
        while (!app.ManiaTitleControlScriptAPI.IsReady) yield();
        app.ManiaTitleControlScriptAPI.PlayMap(fileUrl, "", "");
        mapWatch.setMap(this);
        UI::HideOverlay();
    }

    void drawMapDebug() {
        UI::Text("Debbuging map");
        UI::Text("trackId : " + trackId);
        UI::Text("trackUid : " + trackUid);
        UI::Text("campaignId : " + campaignId);
        UI::Text("name : " + name);
        UI::Text("author : " + author);
        UI::Text("fileUrl : " + fileUrl);
        UI::Text("medals : [" + medals[0] + ", " + medals[1] + ", " + medals[2] + ", " + medals[3] + "]");
        UI::Text("time : " + time);
        UI::Text("difficulty : " + difficulty);

        UI::Separator();

        UI::Text("getMedal() : " + getMedal());
        UI::Text("getDifficulty() : " + getDifficulty());
    }
}

namespace mapInfo {
    mapInfo@ readFromBuffer(MemoryBuffer@ buf) {
        string trackId = StringFromBuffer(buf);
        string trackUid = StringFromBuffer(buf);
        string campaignId = StringFromBuffer(buf);
        string name = StringFromBuffer(buf);
        string author = StringFromBuffer(buf);
        string fileUrl = StringFromBuffer(buf);
        uint64[] medals = uint64ArrayFromBuffer(buf);
        uint64 time = uint64FromBuffer(buf);
        return mapInfo(trackId, trackUid, campaignId, name, author, fileUrl, medals, time);
    }
}